@@ -25,7 +25,6 @@ from pulpcore.plugin.serializers import (
     DistributionSerializer,
     RepositoryVersionRelatedField,
     validate_unknown_fields,
-    TaskSerializer,
 )
 from pulpcore.plugin.util import get_url
 from rest_framework.exceptions import ValidationError
@@ -137,13 +136,14 @@ class AnsibleRepositorySerializer(RepositorySerializer):
     last_synced_metadata_time = serializers.DateTimeField(
         help_text=_("Last synced metadata time."), allow_null=True, required=False
     )
-    last_sync_task = TaskSerializer(read_only=True)
     gpgkey = serializers.CharField(
         help_text="Gpg public key to verify collection signatures against",
         required=False,
         allow_null=True,
     )
 
+    last_sync_task = serializers.SerializerMethodField()
+
     class Meta:
         fields = RepositorySerializer.Meta.fields + (
             "last_synced_metadata_time",
@@ -153,6 +153,12 @@ class AnsibleRepositorySerializer(RepositorySerializer):
         )
         model = AnsibleRepository
 
+    def get_last_sync_task(self, obj):
+        if hasattr(obj, "last_sync_task"):
+            return obj.last_sync_task
+
+        return None
+
 
 class AnsibleRepositorySyncURLSerializer(RepositorySyncURLSerializer):
     """
@@ -225,7 +231,7 @@ class CollectionRemoteSerializer(RemoteSerializer):
         default=False,
     )
 
-    last_sync_task = TaskSerializer(read_only=True)
+    last_sync_task = serializers.SerializerMethodField()
 
     def validate(self, data):
         """
@@ -271,6 +277,12 @@ class CollectionRemoteSerializer(RemoteSerializer):
 
         return data
 
+    def get_last_sync_task(self, obj):
+        if hasattr(obj, "last_sync_task"):
+            return obj.last_sync_task
+
+        return None
+
     class Meta:
         fields = RemoteSerializer.Meta.fields + (
             "requirements_file",
@@ -323,15 +335,6 @@ class CollectionOneShotSerializer(serializers.Serializer):
         default=None,
     )
 
-    def validate(self, data):
-        """Ensure duplicate artifact isn't uploaded."""
-        data = super().validate(data)
-        sha256 = data["file"].hashers["sha256"].hexdigest()
-        artifact = Artifact.objects.filter(sha256=sha256).first()
-        if artifact:
-            raise ValidationError(_("Artifact already exists"))
-        return data
-
 
 class AnsibleDistributionSerializer(DistributionSerializer):
     """
@@ -373,9 +376,11 @@ class TagSerializer(serializers.ModelSerializer):
     A serializer for the Tag model.
     """
 
+    count = serializers.IntegerField(read_only=True)
+
     class Meta:
         model = Tag
-        fields = ["name"]
+        fields = ["name", "count"]
 
 
 class TagNestedSerializer(ModelSerializer):
@@ -477,13 +482,6 @@ class CollectionVersionUploadSerializer(SingleArtifactContentUploadSerializer):
             data["expected_name"] = collection.name
             data["expected_version"] = collection.version
 
-        if CollectionVersion.objects.filter(**{f: data[f"expected_{f}"] for f in fields}).exists():
-            raise ValidationError(
-                _("Collection {}.{}-{} already exists").format(
-                    data["expected_namespace"], data["expected_name"], data["expected_version"]
-                )
-            )
-
         # Super will call deferred_validate on second call in task context
         return super().validate(data)
 
@@ -504,9 +502,31 @@ class CollectionVersionUploadSerializer(SingleArtifactContentUploadSerializer):
         # repository field clashes
         collection_info["origin_repository"] = collection_info.pop("repository", None)
         data.update(collection_info)
+        # `retrieve` needs artifact, but it won't be in validated_data because of `get_artifacts`
+        self.context["artifact"] = artifact
 
         return data
 
+    def retrieve(self, validated_data):
+        """Reuse existing CollectionVersion if provided artifact matches."""
+        namespace = validated_data["namespace"]
+        name = validated_data["name"]
+        version = validated_data["version"]
+        artifact = self.context["artifact"]
+        # TODO switch this check to use digest when ColVersion uniqueness constraint is changed
+        col = CollectionVersion.objects.filter(
+            namespace=namespace, name=name, version=version
+        ).first()
+        if col:
+            if col._artifacts.get() != artifact:
+                raise ValidationError(
+                    _("Collection {}.{}-{} already exists with a different artifact").format(
+                        namespace, name, version
+                    )
+                )
+
+        return col
+
     def create(self, validated_data):
         """Final step in creating the CollectionVersion."""
         tags = validated_data.pop("tags")
@@ -725,21 +745,12 @@ class CollectionVersionSignatureSerializer(NoArtifactContentUploadSerializer):
         allow_null=True,
     )
 
-    def __init__(self, *args, **kwargs):
-        """Ensure `file` field is required."""
-        super().__init__(*args, **kwargs)
-        self.fields["file"].required = True
-
-    def validate(self, data):
+    def deferred_validate(self, data):
         """
         Verify the signature is valid before creating it.
         """
-        data = super().validate(data)
-
-        if "request" not in self.context:
-            # Validate is called twice, first on the viewset, and second on the create task
-            # data should be set up properly on the second time, when request isn't in context
-            data = verify_signature_upload(data)
+        data = super().deferred_validate(data)
+        data = verify_signature_upload(data)
 
         return data
 
