@@ -10,12 +10,12 @@ from urllib.parse import urljoin
 from uuid import uuid4
 
 import yaml
-from aiohttp.client_exceptions import ClientResponseError
+from aiohttp.client_exceptions import ClientError, ClientResponseError
 from asgiref.sync import sync_to_async
 from async_lru import alru_cache
 from django.conf import settings
 from django.db import transaction
-from django.db.models import Q
+from django.db.models import F, Q
 from django.db.utils import IntegrityError
 from django.urls import reverse
 from django.utils.dateparse import parse_datetime
@@ -344,9 +344,8 @@ def create_collection_from_importer(importer_result, metadata_only=False):
             tag, created = Tag.objects.get_or_create(name=name)
             collection_version.tags.add(tag)
 
-        _update_highest_version(collection_version)
-
         collection_version.save()  # Save the FK updates
+        _update_highest_version(collection_version)
     return collection_version
 
 
@@ -421,53 +420,32 @@ def _get_backend_storage_url(artifact_file):
     return url
 
 
-def _update_highest_version(collection_version, save=False):
+def _update_highest_version(collection_version):
     """
     Checks if this version is greater than the most highest one.
 
-    If this version is the first version in collection, is_highest is set to True.
-    If this version is greater than the highest version in collection, set is_highest
-    equals False on the last highest version and True on this version.
-    Otherwise does nothing.
+    TODO: This function violates content immutability. It must be deprecated.
     """
 
-    def is_new_highest(new, old):
-        if bool(new.prerelease) == bool(old.prerelease):
-            return new > old
-        return bool(old.prerelease)
-
-    # did we have one set previously for this collection?
-    last_highest = collection_version.collection.versions.filter(is_highest=True).first()
-
-    if not last_highest:
-        # we only have one version, so mark it as the highest
-        if collection_version.collection.versions.count() == 1:
-            collection_version.is_highest = True
-            if save:
-                collection_version.save(update_fields=["is_highest"])
+    qs = collection_version.collection.versions.annotate(prerelease=Q(version_prerelease__ne=""))
+    highest_subq = qs.order_by("prerelease", "-version")[0:1].values("pk")
+    update_qs = qs.annotate(new_is_highest=Q(pk=highest_subq))
+    # To avoid ordering issues, set everything to False first, then update with proper value
+    for i in range(2):
+        try:
+            with transaction.atomic():
+                update_qs.update(is_highest=False)
+                update_qs.update(is_highest=F("new_is_highest"))
+        except IntegrityError:
+            log.debug(f"Retrying update_highest_version {collection_version.relative_path}")
+            pass
+        else:
             return
-
-        # compute highest from the whole list ...
-        highest = None
-        for cv in collection_version.collection.versions.all():
-            sv = Version(cv.version)
-            if highest is None or is_new_highest(sv, highest[0]):
-                highest = (sv, cv)
-
-        highest[1].is_highest = True
-        if highest[1] != collection_version or save:
-            highest[1].save()
-        return
-
-    # exit if the new CV is not higher
-    if not is_new_highest(Version(collection_version.version), Version(last_highest.version)):
-        return
-
-    last_highest.is_highest = False
-    last_highest.save(update_fields=["is_highest"])
-    collection_version.is_highest = True
-    if save:
-        collection_version.save(update_fields=["is_highest"])
+    log.warning(
+        _("Failed to update is_highest for {}, potentially out of date").format(
+            collection_version.relative_path
+        )
+    )
 
 
 class AnsibleDeclarativeVersion(DeclarativeVersion):
@@ -496,7 +474,7 @@ class AnsibleDeclarativeVersion(DeclarativeVersion):
             ArtifactSaver(),
             QueryExistingContents(),
             DocsBlobDownloader(),
-            CollectionContentSaver(new_version),
+            AnsibleContentSaver(new_version),
             RemoteArtifactSaver(),
             ResolveContentFutures(),
         ]
@@ -719,15 +697,15 @@ class CollectionSyncFirstStage(Stage):
             else:
                 namespace["links"] = dict()
 
-            for key in ("pulp_href", "groups", "id", "related_fields"):
+            for key in ("pulp_href", "groups", "id", "related_fields", "users"):
                 namespace.pop(key, None)
 
             url = namespace.pop("avatar_url", None)
 
             da = (
                 [
-                    DeclarativeArtifact(
-                        Artifact(sha256=namespace["avatar_sha256"]),
+                    DeclarativeFailsafeArtifact(
+                        Artifact(sha256=namespace.get("avatar_sha256")),
                         url=url,
                         remote=self.remote,
                         relative_path=f"{name}-avatar",
@@ -1074,6 +1052,25 @@ class CollectionSyncFirstStage(Stage):
             pb.total = pb.done
 
 
+class DeclarativeFailsafeArtifact(DeclarativeArtifact):
+    """
+    Special handling for downloading Namespace Avatar Artifacts.
+    """
+
+    async def download(self):
+        """Allow download to fail, but not stop the sync."""
+        artifact_copy = self.artifact
+        try:
+            return await super().download()
+        except ClientError as e:
+            # Reset DA so that future stages can properly handle it
+            self.artifact = artifact_copy
+            self.deferred_download = True
+            name = self.extra_data.get("namespace")
+            log.info(f"Failed to download namespace avatar: {name} - {e}, Skipping")
+            return None
+
+
 class DocsBlobDownloader(ArtifactDownloader):
     """
     Stage for downloading docs_blob.
@@ -1114,7 +1111,7 @@ class DocsBlobDownloader(ArtifactDownloader):
         return downloaded
 
 
-class CollectionContentSaver(ContentSaver):
+class AnsibleContentSaver(ContentSaver):
     """
     A modification of ContentSaver stage that additionally saves Ansible plugin specific items.
 
@@ -1149,6 +1146,15 @@ class CollectionContentSaver(ContentSaver):
                 name = d_content.content.name
                 namespace, created = AnsibleNamespace.objects.get_or_create(name=name)
                 d_content.content.namespace = namespace
+                if d_content.d_artifacts:
+                    da = d_content.d_artifacts[0]
+                    # Check to see if avatar failed to download, update metadata if so
+                    if da.deferred_download:
+                        d_content.d_artifacts = None
+                        d_content.content.avatar_sha256 = None
+                    # Check to see if upstream didn't have avatar_sha256 set
+                    elif d_content.content.avatar_sha256 is None:
+                        d_content.content.avatar_sha256 = da.artifact.sha256
 
     def _post_save(self, batch):
         """
@@ -1209,6 +1215,5 @@ class CollectionContentSaver(ContentSaver):
                         continue
                     setattr(collection_version, attr_name, attr_value)
 
-                _update_highest_version(collection_version)
-
                 collection_version.save()
+                _update_highest_version(collection_version)
