@@ -3,6 +3,7 @@ from gettext import gettext as _
 from django.contrib.postgres.search import SearchQuery
 from django.db.models import fields as db_fields
 from django.db.models.expressions import F, Func
+from django.db.models import Count
 from django_filters import filters
 from django.http import HttpResponseRedirect, HttpResponseNotFound
 from drf_spectacular.utils import extend_schema
@@ -36,6 +37,7 @@ from pulpcore.plugin.viewsets import (
 )
 from pulpcore.plugin.util import extract_pk, raise_for_unknown_content_units, get_artifact_url
 from pulp_ansible.app.galaxy.mixins import UploadGalaxyCollectionMixin
+from pulp_ansible.app.utils import get_queryset_annotated_with_last_sync_task
 from .models import (
     AnsibleCollectionDeprecated,
     AnsibleDistribution,
@@ -408,11 +410,10 @@ class AnsibleNamespaceViewSet(ReadOnlyContentViewSet):
     @action(detail=True, methods=["get"], serializer_class=None)
     def avatar(self, request, pk):
         """
-        Dispatches a collection version rebuild task.
+        Tries to find a redirect link to the Namespace's avatar
         """
         ns = self.get_object()
-        artifact = ns.avatar_artifact
-        if artifact:
+        if artifact := ns.avatar_artifact:
             return HttpResponseRedirect(get_artifact_url(artifact))
 
         return HttpResponseNotFound()
@@ -448,7 +449,7 @@ class RoleRemoteViewSet(RemoteViewSet, RolesMixin):
                 "condition": "has_model_or_obj_perms:ansible.delete_roleremote",
             },
             {
-                "action": ["update", "partial_update"],
+                "action": ["update", "partial_update", "set_label", "unset_label"],
                 "principal": "authenticated",
                 "effect": "allow",
                 "condition": "has_model_or_obj_perms:ansible.change_roleremote",
@@ -513,7 +514,7 @@ class GitRemoteViewSet(RemoteViewSet, RolesMixin):
                 "condition": "has_model_or_obj_perms:ansible.delete_gitremote",
             },
             {
-                "action": ["update", "partial_update"],
+                "action": ["update", "partial_update", "set_label", "unset_label"],
                 "principal": "authenticated",
                 "effect": "allow",
                 "condition": "has_model_or_obj_perms:ansible.change_gitremote",
@@ -555,6 +556,14 @@ class AnsibleRepositoryViewSet(RepositoryViewSet, ModifyRepositoryActionMixin, R
     queryset = AnsibleRepository.objects.all()
     serializer_class = AnsibleRepositorySerializer
 
+    def get_queryset(self):
+        qs = super().get_queryset()
+        action = getattr(self, "action", "")
+        if action == "list" or action == "retrieve":
+            qs = get_queryset_annotated_with_last_sync_task(qs)
+
+        return qs
+
     queryset_filtering_required_permission = "ansible.view_ansiblerepository"
 
     DEFAULT_ACCESS_POLICY = {
@@ -577,7 +586,7 @@ class AnsibleRepositoryViewSet(RepositoryViewSet, ModifyRepositoryActionMixin, R
                 "condition": "has_model_or_obj_perms:ansible.delete_ansiblerepository",
             },
             {
-                "action": ["update", "partial_update"],
+                "action": ["update", "partial_update", "set_label", "unset_label"],
                 "principal": "authenticated",
                 "effect": "allow",
                 "condition": [
@@ -988,6 +997,14 @@ class CollectionRemoteViewSet(RemoteViewSet, RolesMixin):
     serializer_class = CollectionRemoteSerializer
     filterset_class = CollectionRemoteFilter
 
+    def get_queryset(self):
+        qs = super().get_queryset()
+        action = getattr(self, "action", "")
+        if action == "list" or action == "retrieve":
+            qs = get_queryset_annotated_with_last_sync_task(qs)
+
+        return qs
+
     queryset_filtering_required_permission = "ansible.view_collectionremote"
 
     DEFAULT_ACCESS_POLICY = {
@@ -1010,7 +1027,7 @@ class CollectionRemoteViewSet(RemoteViewSet, RolesMixin):
                 "condition": "has_model_or_obj_perms:ansible.delete_collectionremote",
             },
             {
-                "action": ["update", "partial_update"],
+                "action": ["update", "partial_update", "set_label", "unset_label"],
                 "principal": "authenticated",
                 "effect": "allow",
                 "condition": "has_model_or_obj_perms:ansible.change_collectionremote",
@@ -1136,7 +1153,7 @@ class AnsibleDistributionViewSet(DistributionViewSet, RolesMixin):
                 ],
             },
             {
-                "action": ["update", "partial_update"],
+                "action": ["update", "partial_update", "set_label", "unset_label"],
                 "principal": "authenticated",
                 "effect": "allow",
                 "condition": [
@@ -1180,6 +1197,10 @@ class TagViewSet(NamedModelViewSet, mixins.ListModelMixin):
     queryset = Tag.objects.all()
     serializer_class = TagSerializer
 
+    def get_queryset(self):
+        qs = super().get_queryset()
+        return qs.annotate(count=Count("ansible_collectionversion"))
+
 
 class CopyViewSet(viewsets.ViewSet):
     """
