@@ -1,75 +1,60 @@
 """Tests related to sync ansible plugin collection content type."""
 from tempfile import NamedTemporaryFile
-from pulp_smash import api, config
 from pulp_smash.pulp3.bindings import PulpTestCase, delete_orphans, monitor_task
-from pulp_smash.pulp3.utils import gen_repo, sync
 from pulp_smash.utils import http_get
 
 from pulpcore.client.pulp_ansible import ContentCollectionVersionsApi
-from pulpcore.client.pulp_ansible.exceptions import ApiException
 
 
 from pulp_ansible.tests.functional.constants import (
-    ANSIBLE_COLLECTION_REMOTE_PATH,
-    ANSIBLE_COLLECTION_VERSION_CONTENT_PATH,
     ANSIBLE_DEMO_COLLECTION_REQUIREMENTS as DEMO_REQUIREMENTS,
-    ANSIBLE_REPO_PATH,
     GALAXY_ANSIBLE_BASE_URL,
 )
-from pulp_ansible.tests.functional.utils import gen_ansible_client, gen_ansible_remote, skip_if
+from pulp_ansible.tests.functional.utils import gen_ansible_client, skip_if
 from pulp_ansible.tests.functional.utils import set_up_module as setUpModule  # noqa:F401
 
 
-class ListContentVersionsCase(PulpTestCase):
-    """Test listing CollectionVersions."""
+def test_tags_filter(
+    ansible_collection_version_api_client,
+    ansible_collection_remote_factory,
+    ansible_sync_factory,
+):
+    """Filter CollectionVersions by using the tags filter.
 
-    @classmethod
-    def setUpClass(cls):
-        """Create class-wide variables."""
-        cls.cfg = config.get_config()
-        cls.client = api.Client(cls.cfg)
+    This test targets the following issue:
 
-    def test_tags_filter(self):
-        """Filter CollectionVersions by using the tags filter.
+    * `Pulp #5571 <https://pulp.plan.io/issues/5571>`_
 
-        This test targets the following issue:
+    Do the following:
 
-        * `Pulp #5571 <https://pulp.plan.io/issues/5571>`_
+    1. Create a repository, and a remote.
+    2. Sync the remote.
+    3. Attempt to filter the CollectionVersions by different tags
 
-        Do the following:
+    Note that the testing.k8s_demo_collection collection has tags 'k8s' and 'kubernetes'.
+    """
+    ansible_sync_factory(
+        remote=ansible_collection_remote_factory(
+            url=GALAXY_ANSIBLE_BASE_URL, requirements_file=DEMO_REQUIREMENTS
+        ).pulp_href
+    )
 
-        1. Create a repository, and a remote.
-        2. Sync the remote.
-        3. Attempt to filter the CollectionVersions by different tags
+    # filter collection versions by tags
+    params = {"tags": "nada"}
+    collections = ansible_collection_version_api_client.list(**params).results
+    assert len(collections) == 0, collections
 
-        Note that the testing.k8s_demo_collection collection has tags 'k8s' and 'kubernetes'.
-        """
-        repo = self.client.post(ANSIBLE_REPO_PATH, gen_repo())
-        self.addCleanup(self.client.delete, repo["pulp_href"])
+    params = {"tags": "k8s"}
+    collections = ansible_collection_version_api_client.list(**params).results
+    assert len(collections) == 1, collections
 
-        body = gen_ansible_remote(url=GALAXY_ANSIBLE_BASE_URL, requirements_file=DEMO_REQUIREMENTS)
-        remote = self.client.post(ANSIBLE_COLLECTION_REMOTE_PATH, body)
-        self.addCleanup(self.client.delete, remote["pulp_href"])
+    params = {"tags": "k8s,kubernetes"}
+    collections = ansible_collection_version_api_client.list(**params).results
+    assert len(collections) == 1, collections
 
-        # Sync the repository.
-        sync(self.cfg, remote, repo)
-
-        # filter collection versions by tags
-        params = {"tags": "nada"}
-        collections = self.client.get(ANSIBLE_COLLECTION_VERSION_CONTENT_PATH, params=params)
-        self.assertEqual(len(collections), 0, collections)
-
-        params = {"tags": "k8s"}
-        collections = self.client.get(ANSIBLE_COLLECTION_VERSION_CONTENT_PATH, params=params)
-        self.assertEqual(len(collections), 1, collections)
-
-        params = {"tags": "k8s,kubernetes"}
-        collections = self.client.get(ANSIBLE_COLLECTION_VERSION_CONTENT_PATH, params=params)
-        self.assertEqual(len(collections), 1, collections)
-
-        params = {"tags": "nada,k8s"}
-        collections = self.client.get(ANSIBLE_COLLECTION_VERSION_CONTENT_PATH, params=params)
-        self.assertEqual(len(collections), 0, collections)
+    params = {"tags": "nada,k8s"}
+    collections = ansible_collection_version_api_client.list(**params).results
+    assert len(collections) == 0, collections
 
 
 class ContentUnitTestCase(PulpTestCase):
@@ -173,9 +158,9 @@ class ContentUnitTestCase(PulpTestCase):
         self.assertIn(msg, exc.exception.args[0])
 
     @skip_if(bool, "content_unit", False)
-    def test_05_duplicate_raise_error(self):
+    def test_05_duplicate_no_error(self):
         """Attempt to create duplicate collection."""
         attrs = dict(namespace="pulp", name="squeezer", version="0.0.9")
-        with self.assertRaises(ApiException) as ctx:
-            self.upload_collection(**attrs)
-        self.assertIn("Collection pulp.squeezer-0.0.9 already exists", ctx.exception.body)
+        response = self.upload_collection(**attrs)
+        created_resources = monitor_task(response.task).created_resources
+        self.assertEqual(created_resources[0], self.content_unit["pulp_href"])
